using ConformalPrediction
using Documenter

ex_meta = quote
    # Import module(s):
    using ConformalPrediction
    using MLJ
    using MLJModelInterface
    const MMI = MLJModelInterface

    # Data:
    X, y = MLJ.make_regression(1000, 2)
    train, calibration, test = partition(eachindex(y), 0.4, 0.4)

    # Model:
    DecisionTreeRegressor = @load DecisionTreeRegressor pkg=DecisionTree
    model = DecisionTreeRegressor()   
end

DocMeta.setdocmeta!(ConformalPrediction, :DocTestSetup, ex_meta; recursive=true)

makedocs(;
    modules=[ConformalPrediction],
    authors="Patrick Altmeyer",
    repo="https://github.com/pat-alt/ConformalPrediction.jl/blob/{commit}{path}#{line}",
    sitename="ConformalPrediction.jl",
    format=Documenter.HTML(;
        prettyurls=get(ENV, "CI", "false") == "true",
        canonical="https://pat-alt.github.io/ConformalPrediction.jl",
        edit_link="main",
        assets=String[],
    ),
    pages=[
        "🏠 Home" => "index.md",
        "🤔 Tutorials" => [
            "Classification" => "classification.md",
            "Regression" => "regression.md",
        ],
        "📖 Reference" => "reference.md",
        "🛠 Contribute" => "contribute.md",
    ],
)

deploydocs(;
    repo="github.com/pat-alt/ConformalPrediction.jl",
    devbranch="main",
)
